@@ -25,6 +25,7 @@ from re import split as re_split
 from shutil import copy2, Error as shutil_Error
 import sys
 from itertools import chain
+import fnmatch
 
 try:
     # Python 2.7
@@ -39,8 +40,8 @@ except ImportError:
 logger = logging.getLogger("hashedassets")
 
 
-class AssetHasher(object):
-    def __init__(self, files, output_dir, map_filename, map_name, map_type, rewritestring, map_only=False, reference=None):
+class AssetMap(object):
+    def __init__(self, files, output_dir, name, format, reference, excludes):
         logger.debug('Incoming files: %s', files)
 
         basedir = commonprefix(files)
@@ -56,25 +57,44 @@ class AssetHasher(object):
         self.output_dir = output_dir
         logger.debug('Output dir is "%s"', self.output_dir)
 
-        relative_files = [relpath(path, self.basedir) for path in chain.from_iterable(list(map(glob, files)))]
+        globfiles = list(chain.from_iterable(list(map(glob, files))))
 
-        logger.debug('Relative files: %s', relative_files)
+        logger.debug("Globfiles: %s", globfiles)
 
-        for file_or_dir in relative_files:
-            for walkroot, _, walkfiles in walk(join(self.basedir, file_or_dir)):
+        for file_or_dir in globfiles:
+            for walkroot, _, walkfiles in walk(file_or_dir):
                 for walkfile in walkfiles:
-                    relative_files.append(normpath(join(relpath(walkroot, self.basedir), walkfile)))
+                    globfiles.append(join(walkroot, walkfile))
 
-        logger.debug('Resolved subdir files: %s', relative_files)
+        logger.debug('Resolved globfiles: %s', globfiles)
 
-        self.files = OrderedDict.fromkeys(relative_files)
+        for exclude in (excludes or []):
 
-        logger.debug("Initialized map, is now %s", self.files)
+            if exclude[-1] is not '*':
+                exclude += '*'
 
-        self.map_filename = map_filename
-        self.map_name = map_name
-        self.map_type = map_type
-        self.map_only = map_only
+            evicts = fnmatch.filter(globfiles, exclude)
+            logger.debug("exclude '%s' evicts => %s", exclude, evicts)
+
+            globfiles = [ globfile for globfile in globfiles if globfile not in evicts ]
+
+        relative_files = [
+            r for r in [
+                relpath(globfile, self.basedir)
+                for globfile
+                in globfiles
+            ]
+            if r is not '.'
+        ]
+
+        logger.debug('Resolved relative files: %s', relative_files)
+
+        self._files = OrderedDict.fromkeys(relative_files)
+
+        logger.debug("Initialized map, is now %s", self._files)
+
+        self.name = name
+        self.format = format
 
         if not reference:
             self.refdir = self.output_dir
@@ -83,32 +103,92 @@ class AssetHasher(object):
         else:
             self.refdir = reference
 
+    def __setitem__(self, name, item):
+        self._files[name] = item
+
+    def __getitem__(self, item):
+        return self._files[item]
+
+    def __iter__(self):
+        for file in self._files:
+            yield file
+
+    def items(self):
+        return self._files.items()
+
+    def read(self, filename):
+        if not filename:
+            return
+
+        if not exists(filename):
+            return
+
+        content = open(filename).read()
+
+        deserialized = SERIALIZERS[self.format].deserialize(content)
+
+        for filename, hashed_filename in list(deserialized.items()):
+            hashed_filename = relpath(join(self.refdir, hashed_filename), self.output_dir)
+            filename = relpath(join(self.refdir, filename), self.output_dir)
+            self[filename] = hashed_filename
+
+        logger.debug("Read map, is now: %s", self._files)
+
+    def write(self, filename):
+        if not filename:
+            return
+
+        newmap = OrderedDict()
+
+        for origin, target in self.items():
+            if target != None:
+                origin = relpath(join(self.output_dir, origin), self.refdir)
+                target = relpath(join(self.output_dir, target), self.refdir)
+                newmap[origin] = target
+
+        serialized = SERIALIZERS[self.format].serialize(newmap, self.name)
+
+        if filename == '-':
+            outfile = sys.stdout
+        else:
+            outfile = open(filename, 'w')
+
+        outfile.write(serialized)
+
+        if filename != '-':
+            outfile.close()
+
+
+class AssetHasher(object):
+    def __init__(self, assetmap, rewritestring, map_only):
+        self.assetmap = assetmap
         self.rewritestring = rewritestring
+        self.map_only = map_only
 
     def process_file(self, filename):
         logger.debug("Processing file '%s'", filename)
 
         try:
             hashed_filename = self.rewritestring % Rewriter(
-              filename, self.basedir)
+              filename, self.assetmap.basedir)
         except IOError as e:
             logger.debug("'%s' does not exist, can't be hashed", filename, exc_info=e)
             return
 
         logger.debug("Determined new hashed filename: '%s'", hashed_filename)
 
-        if self.files[filename]:
+        if self.assetmap[filename]:
             logger.debug("File has been processed in a previous run (hashed to '%s' then)",
-                    self.files[filename])
+                    self.assetmap[filename])
 
-            outfile = join(self.output_dir, self.files[filename])
+            outfile = join(self.assetmap.output_dir, self.assetmap[filename])
 
             if exists(outfile):
                 logger.debug("%s still exists", outfile)
 
-                if hashed_filename == self.files[filename]:
+                if hashed_filename == self.assetmap[filename]:
                     # skip file
-                    logger.debug("Skipping file '%s' -> '%s'", filename, self.files[filename])
+                    logger.debug("Skipping file '%s' -> '%s'", filename, self.assetmap[filename])
                     return
 
                 # remove dangling file
@@ -117,8 +197,8 @@ class AssetHasher(object):
                     logger.info("rm '%s'", outfile)
 
 
-        infile = join(self.basedir, filename).replace('/./', '/')
-        outfile = join(self.output_dir, hashed_filename).replace('/./', '/')
+        infile = join(self.assetmap.basedir, filename).replace('/./', '/')
+        outfile = join(self.assetmap.output_dir, hashed_filename).replace('/./', '/')
 
         try:
             if samefile(infile, outfile):
@@ -142,67 +222,25 @@ class AssetHasher(object):
             # create parent dirs that are needed for the output file
             logger.debug(hashed_filename)
             create_dir, _ = path_split(hashed_filename)
-            logger.info("mkdir -p %s" % join(self.output_dir, create_dir))
-            makedirs(join(self.output_dir, create_dir))
+            logger.info("mkdir -p %s" % join(self.assetmap.output_dir, create_dir))
+            makedirs(join(self.assetmap.output_dir, create_dir))
 
             # try again
             copy2(infile, outfile)
 
-        self.files[filename] = hashed_filename
+        self.assetmap[filename] = hashed_filename
 
         if not self.map_only:
             logger.info("cp '%s' '%s'", infile, outfile)
 
     def process_all_files(self):
-        for f in self.files:
+        for f in self.assetmap:
             self.process_file(f)
 
-    def read_map(self):
-        if not self.map_filename:
-            return
-
-        if not exists(self.map_filename):
-            return
-
-        content = open(self.map_filename).read()
-
-        deserialized = SERIALIZERS[self.map_type].deserialize(content)
-
-        for filename, hashed_filename in list(deserialized.items()):
-            hashed_filename = relpath(join(self.refdir, hashed_filename), self.output_dir)
-            filename = relpath(join(self.refdir, filename), self.output_dir)
-            self.files[filename] = hashed_filename
-
-        logger.debug("Read map, is now: %s", self.files)
-
-    def write_map(self):
-        if not self.map_filename:
-            return
-
-        newmap = OrderedDict()
-
-        for origin, target in list(self.files.items()):
-            if target != None:
-                origin = relpath(join(self.output_dir, origin), self.refdir)
-                target = relpath(join(self.output_dir, target), self.refdir)
-                newmap[origin] = target
-
-        serialized = SERIALIZERS[self.map_type].serialize(newmap, self.map_name)
-
-        if self.map_filename == '-':
-            outfile = sys.stdout
-        else:
-            outfile = open(self.map_filename, 'w')
-
-        outfile.write(serialized)
-
-        if self.map_filename != '-':
-            outfile.close()
-
-    def run(self):
-        self.read_map()
+    def run(self, filename):
+        self.assetmap.read(filename)
         self.process_all_files()
-        self.write_map()
+        self.assetmap.write(filename)
 
 def main(args=None):
     if args == None:
@@ -239,7 +277,7 @@ def main(args=None):
       "-t",
       "--map-type",
       choices=list(SERIALIZERS.keys()),
-      dest="map_type",
+      dest="map_format",
       help=("type of the map. one of "
           + ", ".join(list(SERIALIZERS.keys()))
           + " [default: guessed from MAPFILE]"),
@@ -313,6 +351,16 @@ def main(args=None):
       help="Paths in map will be relative to this directory",
     )
 
+    parser.add_option(
+      "-x",
+      "--exclude",
+      dest="excludes",
+      default=None,
+      type="string",
+      action="append",
+      help="Excludes these files in the input directory",
+    )
+
     (options, args) = parser.parse_args(args)
 
     if options.identity:
@@ -342,11 +390,11 @@ def main(args=None):
 
     map_filename = args[0]
 
-    if not options.map_type and map_filename:
-        options.map_type = splitext(map_filename)[1].lstrip(".")
+    if not options.map_format and map_filename:
+        options.map_format = splitext(map_filename)[1].lstrip(".")
 
-    if not options.map_type in list(SERIALIZERS.keys()):
-        parser.error("Invalid map type: '%s'" % options.map_type)
+    if not options.map_format in list(SERIALIZERS.keys()):
+        parser.error("Invalid map type: '%s'" % options.map_format)
 
     if options.map_only:
         files = args[1:]
@@ -360,19 +408,19 @@ def main(args=None):
         elif not isdir(output_dir):
             parser.error("Output dir at '%s' is not a directory" % output_dir)
 
+    rewritestring = Rewriter.compute_rewritestring(options.strip_extensions,
+            options.digestlength, options.keep_dirs, options.hashfun)
 
-    rewritestring = Rewriter.compute_rewritestring(options.strip_extensions, options.digestlength, options.keep_dirs, options.hashfun)
-
-    AssetHasher(
+    assetmap = AssetMap(
       files=files,
       output_dir=output_dir,
-      map_filename=map_filename,
-      map_name=options.map_name,
-      map_type=options.map_type,
-      map_only=options.map_only,
-      rewritestring=rewritestring,
+      name=options.map_name,
+      format=options.map_format,
       reference=options.reference,
-    ).run()
+      excludes=options.excludes,
+    )
+
+    AssetHasher(assetmap, rewritestring, options.map_only).run(map_filename)
 
 if __name__ == '__main__':
     main(sys.argv[1:])
